/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * xesam-glib
 * Copyright (C) Mikkel Kamstrup Erlandsen 2007 <mikkel.kamstrup@gmail.com>
 * 
 * xesam-glib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * xesam-glib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with xesam-glib.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include "xesam-g-hit.h"
#include "xesam-g-search.h"
#include "xesam-g-globals-private.h"
#include "xesam-g-utils.h"

/**
 * SECTION:xesam-g-hit
 * @short_description: A single hit mapping the metadata requested from the search engine
 *
 * The #XesamGHit class conveniently maps the fields specified in the originating
 * session's #XesamGSession::hit-fields property.
 *
 * #XesamGHit is available since 1.0
 */

G_DEFINE_TYPE (XesamGHit, xesam_g_hit, G_TYPE_OBJECT);

struct _XesamGHitPrivate {
	GPtrArray		*fields;
	GHashTable		*field_map;
	guint			id;
};

static void
xesam_g_hit_init (XesamGHit *hit)
{
	XesamGHitPrivate *priv;
	
	priv = g_new0 (XesamGHitPrivate, 1);
	
	hit->priv = priv;
}

static void
xesam_g_hit_finalize (GObject *object)
{	
	
	XesamGHit 			*hit;
	XesamGHitPrivate 	*priv;
	
	hit = XESAM_G_HIT (object);
	priv = hit->priv;
	
	/* Free field data which we have assumed responsibility for
	 * from the parent Hits object */
	free_ptr_array_of_values (priv->fields);
	
	g_hash_table_unref (priv->field_map);
	
	g_free (priv);

	G_OBJECT_CLASS (xesam_g_hit_parent_class)->finalize (object);
}

static void
xesam_g_hit_class_init (XesamGHitClass *klass)
{
	GObjectClass* object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = xesam_g_hit_finalize;
}

/**
 * xesam_g_hit_get_field
 * @hit: The hit object on which to look up a field value
 * @field_name: The fully qualified name of the field as set through the session 
 *              property #XesamGSession:hit-fields.
 * @returns: The value of a named field. If the field is not known
 *           %NULL is returned. 
 *
 * Get the value of a field by name look up. This method has the overhead
 * of a hash table look up. For even faster access use
 * xesam_g_hit_get_field_by_id().
 * 
 * Alternatively use xesam_g_hit_get_field_by_id() if you know the
 * array index of the field in the session's hit-fields property.
 *
 * If you know the data type of the field beforehand you can use the convenience
 * method xesam_g_hit_get_string() to access the field value.
 */
const GValue*
xesam_g_hit_get_field (XesamGHit *hit, const gchar *field_name)
{
	guint *field_entry;
	
	field_entry = g_hash_table_lookup (hit->priv->field_map, field_name);
	
	if (field_entry == NULL) {
		g_warning ("Unknow field '%s' requested from Hits object",
				   field_name);
		return NULL;
	}
	
	g_return_val_if_fail (*field_entry >= 0 && 
						  *field_entry < hit->priv->fields->len,
						  NULL);
	
	return g_ptr_array_index (hit->priv->fields, *field_entry);
}

/**
 * xesam_g_hit_get_field_by_id
 * @hit: The hit object on which to look up a field value
 * @field_id: The offset the field in the  session property 
 *            #XesamGSession:hit-fields.
 * @returns: The value of a field given by its offset in the
 *           session's hit-fields property.
 *
 * Look up a field value based on the offset of the field name in the 
 * sessions #XesamGSession:hit-fields property.
 *
 * To look up a field by name use xesam_g_hit_get_field().
 */
const GValue*
xesam_g_hit_get_field_by_id (XesamGHit *hit, guint field_id)
{
	g_return_val_if_fail (field_id >= 0, NULL);
	g_return_val_if_fail (field_id < hit->priv->fields->len, NULL);
	
	return g_ptr_array_index (hit->priv->fields, field_id);
}

/**
 * xesam_g_hit_get_string
 * @hit: The hit to look up a string valued field on
 * @returns: The string value of the field. The memory allocated by the string
 *           is private to the hit and should not be freed.
 *
 * Look up the value of a field containing a string. For generic field value
 * look ups use xesam_g_hit_get_field ().
 */
const gchar*
xesam_g_hit_get_string (XesamGHit	*hit,
						const gchar	*field_name)
{
	const GValue	*value;
	
	g_return_val_if_fail (XESAM_IS_G_HIT (hit), NULL);
	g_return_val_if_fail (field_name != NULL, NULL);
	
	value = xesam_g_hit_get_field (hit, field_name);
	
	if (value == NULL)
		return NULL;
	
	if (G_VALUE_HOLDS (value, G_TYPE_STRING))
		return g_value_get_string (value);
	
	g_warning ("XesamGHit does not contain a string value as requested");
	return NULL;
}

/**
 * xesam_g_hit_get_id
 * @hit: The hit to get the id for
 * @returns: The hit id. This is defined as the offset of the hit relative to
 *           the total number of hits retrieved from the server.
 *
 * The Hit id is the offset of the hit within the scope of all results
 * returned by a #XesamGSearch.
 */
guint
xesam_g_hit_get_id (XesamGHit *hit)
{
	g_return_val_if_fail (XESAM_IS_G_HIT(hit), 0);
	
	return hit->priv->id;
}

/**
 * xesam_g_hit_new
 * @id: The id of the hit to create
 * @field_map: A map from field_name to array offset for the @field_data
 * @field_data: An array of hit data. The array elements map to the fields
 *              specified in the #XesamGSession:hit-fields property via the
 *              @field_map argument to this method.
 * @returns: A newly constructed hit object
 *
 * Package private method declared in xesam-g-globals-private.h
 *
 * The passed %GPtrArray, @field_data, is henceforth considered owned
 * by the newly created Hit object.
 */
XesamGHit*
xesam_g_hit_new (guint		id, 
				 GHashTable *field_map, 
				 GPtrArray *field_data)
{
	XesamGHit 			*hit;
	XesamGHitPrivate 	*priv;
	
	g_return_val_if_fail (field_map != NULL, NULL);
	g_return_val_if_fail (field_data != NULL, NULL);
	
	/* This checck is important since the data comes directly from a
	 * XesamGSearcher which is an untrusted source */
	g_return_val_if_fail (g_hash_table_size(field_map) == field_data->len, NULL);
	
	hit = XESAM_G_HIT (g_object_new(XESAM_TYPE_G_HIT, NULL));
	priv = hit->priv;
	
	priv->id = id;
	priv->fields = field_data;
	priv->field_map = field_map;
	
	g_hash_table_ref (field_map);
	
	return hit;
}

static void
_append_name (gchar *key, guint *value, gchar** data)
{
	// FIXME: We need to check array index bounds!
	data[*value] = g_strdup (key);
}

/**
 * xesam_g_hit_get_field_names:
 * @hit: The hit object for which to list the field names
 * @returns: A %NULL terminated array of strings 
 *
 * Return a NULL terminated array listing the field names
 * used in this Hit. You must free the array with g_strfreev().
 */
gchar**
xesam_g_hit_get_field_names (XesamGHit	*hit)
{
	guint	size;
	gchar	**field_names;
	
	g_return_val_if_fail (XESAM_IS_G_HIT (hit), NULL);
	
	size = g_hash_table_size (hit->priv->field_map);
	field_names = g_new0 (gchar*, size+1);
	
	g_hash_table_foreach (hit->priv->field_map, (GHFunc)_append_name, field_names);
	
	return field_names;
}

/**
 * xesam_g_hit_get_data
 * @hit: The hit object for which to return the data list for
 * @returns: A pointer array containing the raw hit data. This array should not
 *           be modified.
 *
 * Get raw access to the hits field data. The data in the returned array
 * map to the fields specified in #XesamGSession:hit-fields property.
 */
const GPtrArray*
xesam_g_hit_get_data (XesamGHit	*hit)
{
	g_return_val_if_fail (XESAM_IS_G_HIT (hit), NULL);
	
	return hit->priv->fields;
}
