/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * xesam-glib
 * Copyright (C) Mikkel Kamstrup Erlandsen 2007 <mikkel.kamstrup@gmail.com>
 * 
 * xesam-glib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * xesam-glib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with xesam-glib.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include "xesam-g-hits.h"
#include "xesam-g-search.h"
#include "xesam-g-globals-private.h"
#include "xesam-g-utils.h"
#include "xesam-g-debug-private.h"

/**
 * SECTION:xesam-g-hits
 * @short_description: Container used to encapsulate a batch of #XesamGHit objects
 *
 * Container for conveniently handling and managing a batch of #XesamGHit objects.
 *
 * #XesamGHits is available since 1.0
 */

struct _XesamGHitsPrivate {
	gboolean		notify_continue;
	guint			batch_offset;
	guint			count;
	XesamGSearch	*search;
	GPtrArray		*data;
	XesamGHit		**hit_instances;
};

/* Private methods */
static void 	_ensure_continue_search 		(XesamGHits		*hits);

G_DEFINE_TYPE (XesamGHits, xesam_g_hits, G_TYPE_OBJECT);

static void
xesam_g_hits_init (XesamGHits *hits)
{
	XesamGHitsPrivate	*priv;
	
	priv = g_new0(XesamGHitsPrivate, 1);
	priv->notify_continue = TRUE;
	
	hits->priv = priv;
}

static void
xesam_g_hits_finalize (GObject *object)
{
	XesamGHits			*hits;
	XesamGHitsPrivate	*priv;
	XesamGHit			*hit;
	gint				idx;

	hits = XESAM_G_HITS (object);
	priv = hits->priv;
	
	xesam_g_debug_object (hits, "Finalizing");
	
	g_object_unref (priv->search);
	
	/* Free cached hit objects and free unused hit data structures.
	 * Recall that cached Hit objects assume responsibility for the
	 * sub-GPtrArrays of priv->data. */
	for (idx = 0; idx < priv->count; idx++) {
		hit = priv->hit_instances[idx];
		if (hit) {
			if (!G_IS_OBJECT (hit)) {
				g_critical ("While finalizing XesamGHits, child hit object with"
							" id '%u' invalid. It has probably been "
							"unreferenced illegally elsewhere",
							priv->batch_offset + idx);
			} else {
				g_object_unref (hit);
			}
		} else {			
			free_ptr_array_of_values (g_ptr_array_index(priv->data,idx));
		}
			
	}
	
	/* Now free the structures holding the actual hits */
	g_free (priv->hit_instances);
	g_ptr_array_free (priv->data, FALSE);
	
	g_free (priv);
	
	G_OBJECT_CLASS (xesam_g_hits_parent_class)->finalize (object);
}

static void
xesam_g_hits_class_init (XesamGHitsClass *klass)
{
	GObjectClass* object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = xesam_g_hits_finalize;
}

/**
 * xesam_g_hits_get_count
 * @hits: The hits object to get the hit count on
 * @returns: The number of hits in the XesamGHits object
 */
guint
xesam_g_hits_get_count (XesamGHits* hits)
{	
	_ensure_continue_search (hits);
	
	return hits->priv->count;
}

/**
 * xesam_g_hits_get
 * @hits: The #XesamGHits object to retrieve a hit from
 * @offset: Offset in the #XesamGHits of the hit to retrieve
 * @returns: The hit at the specified offset or %NULL if the offset is out of
 *           bounds
 *
 * Get a handle to the n'th #XesamGHit in a #XesamGHits object.
 *
 * #XesamGHit objects are generated lazily and cached. The hit is cached in 
 * the #XesamGHits object for the entire lifetime of the parent hits.
 *
 * If you need a reference to the hit past the lifetime of the #XesamGHits 
 * object's lifetime you need to reference the #XesamGHit object.
 */
XesamGHit*
xesam_g_hits_get (XesamGHits *hits, guint offset)
{
	XesamGHit 			*hit;
	XesamGHitsPrivate 	*priv;
	guint				hit_id;
	GHashTable			*field_map;
	GPtrArray			*field_data;
	
	priv = hits->priv;
	
	_ensure_continue_search (hits);
	
	if (offset >= priv->count) {
		g_critical ("Off set %u out of bounds, "
					"XesamGHits object only contain %u hits",
					offset, priv->count);
		return NULL;
	}
	
	/* Check cache */
	hit = priv->hit_instances[offset];
	if (hit) {
		return hit;
	}
	
	/* Not cached, spawn Hit object on the fly */
	hit_id = priv->batch_offset + offset;
	field_map = xesam_g_search_get_field_map (priv->search);
	field_data = g_ptr_array_index (priv->data, offset);
		
	hit = xesam_g_hit_new (hit_id, field_map, field_data);
	
	hits->priv->hit_instances[offset] = hit;
	
	return hit;
}

static void
_print_value (GValue *value, gpointer data)
{
	g_printf ("Field: %s\n", g_value_get_string (value));
}


static void
_print_hit (GPtrArray *hit_data, gpointer ptr)
{
	g_ptr_array_foreach (hit_data, (GFunc) _print_value, NULL);
	g_printf ("-------\n");
}

/**
 * xesam_g_hits_new
 * @search: Search to create a hits object for
 * @batch_offset: The offset of the hits object relative to the total number
 *                of hits on the entire search
 * @count: Number of hits in the XesamGHits object
 * @hits_data: An array of GPtrArrays with hit data as per XesamGSession:hit-fields
 * @returns: A newly allocated hits object
 *
 * Package private method declared in xesam-g-globals-private.h
 *
 * The Hits object is assumed to take ownership of the passed
 * GPtrArray containing the hit data.
 */
XesamGHits*
xesam_g_hits_new (XesamGSearch *search, 
				  guint batch_offset, 
				  guint count,
				  GPtrArray *hits_data)
{	
	XesamGHits	*hits;
	
	hits = XESAM_G_HITS (g_object_new (XESAM_TYPE_G_HITS, NULL));
	hits->priv->search = search;
	hits->priv->batch_offset = batch_offset;
	hits->priv->count = count;
	hits->priv->hit_instances = g_new0(XesamGHit*, count);
	hits->priv->data = hits_data;
		
	g_object_ref (search);
	
	return hits;
}

/* Make sure that we call xesam_g_search_continue one and only
 * one time per Hits object.
 * All public methods invoke this to make sure we trigger a
 * load of the next batch of pending hits. */
static void
_ensure_continue_search (XesamGHits *hits)
{
	if (hits->priv->notify_continue) {
		xesam_g_debug_object (hits, "Detected interaction. Requesting Search@%p"
							  " to continue hit retrieval", hits->priv->search);
		hits->priv->notify_continue = FALSE;
		xesam_g_search_continue (hits->priv->search);
	}
}
